---------------
-- Purge schema
----------------

DROP VIEW  IF EXISTS image_v;
DROP TABLE IF EXISTS image_t;
DROP INDEX IF EXISTS image_i1;
DROP TABLE IF EXISTS master_dark_t;
DROP TABLE IF EXISTS state_t;
